//
//  Score.swift
//  Tetris
//
//  Created by Santiago Pavón Gómez on 9/8/16.
//  Copyright © 2016 UPM. All rights reserved.
//

import Foundation

protocol ScoreDelegate : class {
    func update(games: Int, level: Int, maxRows: Int,  rows: Int)
}


class Score {
    
    // Delegado al que informar de los cambios en las puntuaciones
    weak var delegate: ScoreDelegate?
    
    // Número de partidas jugadas
    private var games = 0
    
    // Nivel usado para seleccionar la velocidad
    private(set) var level = 0
    
    // Máximo número de filas obtenido
    private var maxRows = 0
    
    // Número de filas completadas en la partida actual
    private var rows = 0
    
    // Número de fila a completar para aumentar level
    static let rowsToIncrementLevel = 5
    
    init() {
        
        load()
    }
    
    
    private func save() {
    
        let ud = UserDefaults.standard
        
        ud.set(games, forKey: "games")
        ud.set(maxRows, forKey: "maxScore")
        
        ud.synchronize()
    }
    
    private func load() {
        
        let ud = UserDefaults.standard
 
        games = ud.integer(forKey: "games")
        maxRows = ud.integer(forKey: "maxScore")
    }
    
    
    // Actualizar el estado para empezar una partida nueva
    func newGame() {
        games += 1
        level = 1
        rows = 0
        
        delegate?.update(games: games, level: level, maxRows: maxRows, rows: rows)
        
        save()
    }
    
    // Actualizar la puntuacion al completarse/eliminarse una fila
    func rowCompleted() {
        rows += 1
        maxRows = max (rows,maxRows)
        if rows % Score.rowsToIncrementLevel == 0 {
            level += 1
        }
        
        save()
        
        delegate?.update(games: games, level: level, maxRows: maxRows, rows: rows)
    }
}
